@@ -38,6 +38,58 @@
 
 using namespace std::chrono_literals;
 
+namespace
+{
+bool isNumericVersion(const QString &value, QStringList *parts = nullptr)
+{
+    if (value.isEmpty()) {
+        return false;
+    }
+
+    QStringList splitParts = value.split('.', Qt::KeepEmptyParts);
+    if (splitParts.isEmpty()) {
+        return false;
+    }
+
+    for (const QString &part : splitParts) {
+        if (part.isEmpty()) {
+            return false;
+        }
+        for (QChar c : part) {
+            if (!c.isDigit()) {
+                return false;
+            }
+        }
+    }
+
+    if (parts != nullptr) {
+        *parts = splitParts;
+    }
+    return true;
+}
+
+bool useCompactTargetVersion(const QString &sourceBase, const QString &targetBase)
+{
+    QStringList sourceParts;
+    QStringList targetParts;
+    if (!isNumericVersion(sourceBase, &sourceParts) || !isNumericVersion(targetBase, &targetParts)) {
+        return false;
+    }
+
+    if (sourceParts.size() != targetParts.size() || sourceParts.size() < 2) {
+        return false;
+    }
+
+    for (qsizetype i = 0; i < sourceParts.size() - 1; ++i) {
+        if (sourceParts.at(i) != targetParts.at(i)) {
+            return false;
+        }
+    }
+
+    return sourceParts.last() != targetParts.last();
+}
+} // namespace
+
 MainWindow::MainWindow(const QString &patchFile, QWidget *parent)
     : QDialog(parent),
       ui(new Ui::MainWindow)
@@ -55,8 +107,8 @@ MainWindow::MainWindow(const QString &patchFile, QWidget *parent)
     }
     ui->tabWidget->setCurrentWidget(ui->tabCreatePatch);
     ui->spinCompressionLevel->setValue(settings.value("compression_level", 4).toInt());
-    ui->comboCompression->setCurrentIndex(
-        ui->comboCompression->findText((settings.value("secondary_compression", "None").toString())));
+    int compIdx = ui->comboCompression->findText(settings.value("secondary_compression", "None").toString());
+    ui->comboCompression->setCurrentIndex(compIdx != -1 ? compIdx : 0);
     if (QFile::exists(patchFile)) {
         ui->tabWidget->setCurrentWidget(ui->tabApplyPatch);
         ui->textApplyPatch->setText(patchFile);
@@ -97,12 +149,8 @@ void MainWindow::cmdDone()
     progress->close();
 }
 
-void MainWindow::onSelectFile(QLineEdit *lineEdit)
+void MainWindow::onSelectFile(QLineEdit *lineEdit, const QString &filter)
 {
-    QString filter = lineEdit->objectName() != "textApplyPatch"
-                         ? tr("ISO Files (*.iso);;All Files (*)", "kinds of files to choose")
-                         : tr("ISO Files (*.xdelta3);;All Files (*)", "kinds of files to choose");
-    // Choosing a file
     QString selected
         = QFileDialog::getOpenFileName(this, tr("Select file", "choose a file"), QDir::currentPath(), filter);
     if (checkFile(selected)) {
@@ -112,7 +160,7 @@ void MainWindow::onSelectFile(QLineEdit *lineEdit)
         checkAllinfo();
         return;
     }
-    if (lineEdit->objectName() == "textTarget") {
+    if (lineEdit == ui->textTarget) {
         setPatchName();
     }
     checkAllinfo();
@@ -128,15 +176,10 @@ void MainWindow::onSelectDir()
         return;
     }
 
-    if (QFile::exists(path)) {
-        if (ui->tabWidget->currentWidget() == ui->tabCreatePatch) {
-            ui->textPatch->setText(path + "/" + name);
-        } else {
-            ui->textOutput->setText(path + "/" + name);
-        }
+    if (ui->tabWidget->currentWidget() == ui->tabCreatePatch) {
+        ui->textPatch->setText(path + "/" + name);
     } else {
-        QMessageBox::warning(this, tr("Path not found", "warning about not finding the target directory"),
-                             tr("Please select directory again.", "message to try again"));
+        ui->textOutput->setText(path + "/" + name);
     }
     checkAllinfo();
 }
@@ -149,11 +192,6 @@ void MainWindow::applyPatch()
         return;
     }
 
-    QString output;
-    if (!ui->textOutput->text().isEmpty()) {
-        output = " \"" + ui->textOutput->text() + '"';
-    }
-
     if (QFileInfo(ui->textOutput->text()).isFile()) {
         if (QMessageBox::No
             == QMessageBox::question(
@@ -165,17 +203,21 @@ void MainWindow::applyPatch()
     progress->show();
     elapsedTimer.restart();
     QString cmdout;
-    bool res = cmd.run("xdelta3 -f decode -s \"" + ui->textInput->text() + "\" \"" + ui->textApplyPatch->text() + '"'
-                            + output,
-                        &cmdout);
+    QStringList args;
+    args << "-f" << "decode" << "-s" << ui->textInput->text() << ui->textApplyPatch->text();
+    if (!ui->textOutput->text().isEmpty()) {
+        args << ui->textOutput->text();
+    }
+    bool res = cmd.run("xdelta3", args, &cmdout);
     QString elapsedStr = formatElapsedTime(elapsedTimer.elapsed());
-    QString location = output.isEmpty() ? QFileInfo(ui->textInput->text()).absolutePath().remove("\"")
-                                        : QFileInfo(output).absolutePath().remove("\"");
+    QString location = ui->textOutput->text().isEmpty()
+                           ? QFileInfo(ui->textInput->text()).absolutePath()
+                           : QFileInfo(ui->textOutput->text()).absolutePath();
 
     if (res) {
         QMessageBox::information(
             this, tr("Success", "information that file was successfully written"),
-            tr("File was successfuly written to '%1' directory.", "information that file was successfully written")
+            tr("File was successfully written to '%1' directory.", "information that file was successfully written")
                     .arg(location)
                 + '\n'
                 + tr("Took %1 to patch the file.", "elapsed time, leave %1 untranslated").arg(elapsedStr));
@@ -213,7 +255,7 @@ bool MainWindow::checkFile(const QString &fileName)
     if (!QFileInfo(fileName).isFile()) {
         QMessageBox::warning(
             this, tr("File not found", "warning about file not found"),
-            tr("File '%1' found or not a file, please double-check the input.", "warning about file not found")
+            tr("File '%1' not found or not a file, please double-check the input.", "warning about file not found")
                 .arg(fileName));
         return false;
     } else {
@@ -223,7 +265,7 @@ bool MainWindow::checkFile(const QString &fileName)
 
 void MainWindow::createPatch()
 {
-    QString force;
+    bool force = false;
     if (QFileInfo(ui->textPatch->text()).isFile()) {
         if (QMessageBox::No
             == QMessageBox::question(
@@ -231,22 +273,26 @@ void MainWindow::createPatch()
                 tr("Delta file exists, do you want to overwrite?", "warning about overwritting file"))) {
             return;
         } else {
-            force = "-f ";
+            force = true;
         }
     }
     progress->show();
     elapsedTimer.restart();
     QString cmdout;
-    bool res = cmd.run("xdelta3 " + force + "encode -" + ui->spinCompressionLevel->cleanText() + " -S "
-                            + ui->comboCompression->currentText().toLower() + " -s \"" + ui->textSource->text() + "\" \""
-                            + ui->textTarget->text() + "\" \"" + ui->textPatch->text() + '"',
-                        &cmdout);
+    QStringList args;
+    if (force) {
+        args << "-f";
+    }
+    args << "encode" << ("-" + ui->spinCompressionLevel->cleanText()) << "-S"
+         << ui->comboCompression->currentText().toLower() << "-s" << ui->textSource->text()
+         << ui->textTarget->text() << ui->textPatch->text();
+    bool res = cmd.run("xdelta3", args, &cmdout);
     QString elapsedStr = formatElapsedTime(elapsedTimer.elapsed());
     if (res) {
-        QString patchPath = QFileInfo(ui->textPatch->text()).absoluteFilePath().remove('"');
+        QString patchPath = QFileInfo(ui->textPatch->text()).absoluteFilePath();
         QString patchSize = formatFileSize(QFileInfo(patchPath).size());
-        QMessageBox::information(this, tr("Success", "information on file written succesfully"),
-                                  tr("File '%1' was successfuly written.", "information on file written succesfully")
+        QMessageBox::information(this, tr("Success", "information on file written successfully"),
+                                  tr("File '%1' was successfully written.", "information on file written successfully")
                                           .arg(patchPath)
                                       + '\n'
                                       + tr("Patch size: %1", "size of the created patch file").arg(patchSize)
@@ -256,7 +302,7 @@ void MainWindow::createPatch()
     } else {
         QMessageBox::critical(
             this, tr("Error"),
-            tr("Error: Could not write the file.", "information that file was not written succsessfully") + "\n\n"
+            tr("Error: Could not write the file.", "information that file was not written successfully") + "\n\n"
                 + cmdout);
     }
 }
@@ -266,11 +312,17 @@ void MainWindow::setConnections()
     connect(ui->pushApplyPatch, &QPushButton::pressed, this, &MainWindow::applyPatch);
     connect(ui->pushCreatePatch, &QPushButton::pressed, this, &MainWindow::createPatch);
     connect(ui->pushPatchLocation, &QPushButton::pressed, this, &MainWindow::onSelectDir);
-    connect(ui->pushSelectInput, &QPushButton::pressed, this, [this] { onSelectFile(ui->textInput); });
+    const QString isoFilter = tr("ISO Files (*.iso);;All Files (*)", "kinds of files to choose");
+    const QString deltaFilter = tr("Delta Files (*.xdelta3);;All Files (*)", "kinds of files to choose");
+    connect(ui->pushSelectInput, &QPushButton::pressed, this,
+            [this, isoFilter] { onSelectFile(ui->textInput, isoFilter); });
     connect(ui->pushSelectOutput, &QPushButton::pressed, this, &MainWindow::onSelectDir);
-    connect(ui->pushSelectPatch, &QPushButton::pressed, this, [this] { onSelectFile(ui->textApplyPatch); });
-    connect(ui->pushSelectSource, &QPushButton::pressed, this, [this] { onSelectFile(ui->textSource); });
-    connect(ui->pushSelectTarget, &QPushButton::pressed, this, [this] { onSelectFile(ui->textTarget); });
+    connect(ui->pushSelectPatch, &QPushButton::pressed, this,
+            [this, deltaFilter] { onSelectFile(ui->textApplyPatch, deltaFilter); });
+    connect(ui->pushSelectSource, &QPushButton::pressed, this,
+            [this, isoFilter] { onSelectFile(ui->textSource, isoFilter); });
+    connect(ui->pushSelectTarget, &QPushButton::pressed, this,
+            [this, isoFilter] { onSelectFile(ui->textTarget, isoFilter); });
     connect(ui->textPatch, &QLineEdit::editingFinished, this, [this] { checkAllinfo(); });
     connect(ui->textSource, &QLineEdit::editingFinished, this, [this] {
         if (checkFile(ui->textSource->text()) && !ui->textTarget->text().isEmpty()) {
@@ -343,6 +395,12 @@ void MainWindow::setPatchName()
         // Source has a suffix (e.g., Blah_blah_beta1) and target is final (e.g., Blah_blah)
         QString suffix = sourceBase.mid(targetBase.length() + 1); // Remove targetBase and underscore
         patchName = targetBase + "-" + suffix + "_to_final.xdelta3";
+    } else if (isNumericVersion(sourceBase) && isNumericVersion(targetBase)) {
+        if (useCompactTargetVersion(sourceBase, targetBase)) {
+            patchName = sourceBase + "_to_" + targetBase.section('.', -1) + ".xdelta3";
+        } else {
+            patchName = sourceBase + "_to_" + targetBase + ".xdelta3";
+        }
     } else {
         QString prefix = findCommonPrefix(sourceBase, targetBase);
         if (prefix.length() < 3) {
@@ -405,7 +463,7 @@ void MainWindow::updateBar()
 {
     Cmd cmd2;
     QString output;
-    if (cmd2.run("progress -c xdelta3 | tail -n2", &output, true)) {
+    if (cmd2.run("progress -c xdelta3 | tail -n2", &output, Cmd::Quiet)) {
         bool ok {false};
         QString percentage = output.trimmed().section("%", 0, 0);
         int prog = static_cast<int>(percentage.toDouble(&ok));
